/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth;

/**
 * Implementors of this interface are allowed to take control over the
 * authentication procedure when properly configured. The life cycle of
 * AuthenticationHandlers is clearly stated: they get <i>initialized</i> then
 * multiple calls to the <i>onLogin</i> and <i>onLogout</i> methods can be
 * performed and finally they get <i>destroyed</i> just before finalization.
 * 
 * @author rlogiacco
 * @author svetrini
 */
public interface AuthenticationHandler extends Handler {

	/**
	 * This method is called upon user access attempts.
	 * 
	 * @param user the user attempting to log into the system.
	 * @param credentials the credentials providen by the user.
	 * @throws Exception thrown if the access should be denied for any reason.
	 */
	public void onLogin(User user, Credentials credentials) throws Exception;

	/**
	 * This method is called upon user disconnection attempts.
	 * 
	 * @param user the user requesting to log out from the system.
	 * @throws Exception thrown if the disconnection has failed for any reason.
	 */
	public void onLogout(User user) throws Exception;
}
