/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth;

import java.util.Map;

/**
 * Implementors of this interface are allowed to take control over the
 * authorization procedure when properly configured. The life cycle of
 * AuthorizationHandlers is clearly stated: they get <i>initialized</i> then
 * multiple calls to the <i>onRequest</i> method can be performed and finally
 * they get <i>destroyed</i> just before finalization.
 * 
 * @author rlogiacco
 * @author svetrini
 */
public interface AuthorizationHandler extends Handler {

	/**
	 * This method is called upon user usage attempts.
	 * 
	 * @param user
	 * @param scope
	 * @param parameters
	 * @return 
	 * @throws AuthorizationException
	 */
	public boolean onRequest(User user, Scope scope, Map parameters);
}
