/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import net.smartlab.config.Configuration;
import net.smartlab.config.ConfigurationException;
import net.smartlab.config.Element;
import net.smartlab.config.XMLConfiguration;
import net.smartlab.web.BusinessException;
import net.smartlab.web.DAOException;
import net.smartlab.web.DataAccessObject.SearchInfo;
import net.smartlab.web.page.CollectionPaginator;

/**
 * TODO documentation
 * 
 * @author svetrini
 * @author rlogiacco
 */
public class Domain extends net.smartlab.web.Domain {

	/**
	 * Stores the singleton <code>instance</code> for this class.
	 * 
	 * @uml.property name="instance"
	 */
	private static Domain instance;

	/**
	 * @uml.property name="sessions"
	 * @uml.associationEnd multiplicity="(0 -1)" ordering="true"
	 *                     elementType="net.smartlab.web.auth.Permission"
	 *                     qualifier="securityToken:java.lang.String
	 *                     net.smartlab.web.auth.User"
	 */
	private static Map sessions = new HashMap();

	/**
	 * TODO documentation
	 * 
	 * @uml.property name="handlers"
	 */
	private Map handlers = new HashMap();

	/**
	 * TODO documentation
	 * 
	 * @uml.property name="scopes"
	 */
	private Map scopes = new HashMap();

	/**
	 * TODO documentation
	 * 
	 * @uml.property name="authenticator"
	 * @uml.associationEnd multiplicity="(0 -1)"
	 *                     elementType="net.smartlab.web.auth.AuthenticationHandler"
	 */
	private Collection authenticator = new ArrayList();

	/**
	 * TODO documentation
	 * 
	 * @uml.property name="authorizator"
	 * @uml.associationEnd multiplicity="(0 -1)"
	 *                     elementType="net.smartlab.web.auth.Domain"
	 */
	private Collection authorizator = new ArrayList();

	/**
	 * TODO documentation
	 * 
	 * @uml.property name="registrator"
	 * @uml.associationEnd multiplicity="(0 -1)"
	 *                     elementType="net.smartlab.web.auth.RegistrationHandler"
	 */
	private Collection registrator = new ArrayList();

	/**
	 * TODO documentation
	 * 
	 * @uml.property name="auditor"
	 * @uml.associationEnd multiplicity="(0 -1)"
	 *                     elementType="net.smartlab.web.auth.Handler"
	 */
	private Collection auditor = new ArrayList();

	private Random random = new Random();


	/**
	 * Blocks class instantiation to enforce the singleton pattern.
	 */
	private Domain() {
		this.init();
	}

	/**
	 * Returns the singleton instance.
	 * 
	 * @return returns the singleton instance.
	 * @uml.property name="instance"
	 */
	public static synchronized Domain getInstance() {
		if (instance == null) {
			instance = new Domain();
		}
		return instance;
	}

	public Configuration getConfiguration() throws ConfigurationException {
		return super.getConfiguration("security.xml");
	}

	/**
	 * TODO documentation
	 */
	protected void init() {
		try {
			Configuration config = this.getConfiguration();
			if (config != null) {
				// configure generic handlers
				Iterator handlers = config.getElement("handlers").getElements().iterator();
				while (handlers.hasNext()) {
					Element handler = (Element)handlers.next();
					this.handlers.put(handler.getAttribute("id"), this.init(handler));
				}
				// configure global handlers
				Iterator globals = config.getElement("global").getElements().iterator();
				while (globals.hasNext()) {
					Element element = (Element)globals.next();
					if (element.getName().equals("authentication")) {
						authenticator.add((AuthenticationHandler)this.init(element));
					} else if (element.getName().equals("authorization")) {
						authorizator.add((AuthorizationHandler)this.init(element));
					} else if (element.getName().equals("registration")) {
						registrator.add((RegistrationHandler)this.init(element));
					} else if (element.getName().equals("audit")) {
						auditor.add((Handler)this.init(element));
					}
				}
				logger.info("global handlers configured");
				//Iterator includes = config.getElements("include").iterator();
				//while (includes.hasNext()) {
					//Element element = (Element)includes.next();
					// call the initialization procedure for each sub module
					//this.init(element.getAttribute("resource"));
				//}
			}
		} catch (Exception e) {
			logger.fatal("Initialization failure", e);
		}
	}

	public void init(String resource) {
		try {
			logger.info("including " + resource);
			Configuration config = new XMLConfiguration(this.getClass().getResource(resource));
			// configure generic handlers
			Iterator handlers = config.getElement("handlers").getElements().iterator();
			while (handlers.hasNext()) {
				Element handler = (Element)handlers.next();
				this.handlers.put(handler.getAttribute("id"), this.init(handler));
			}
			// configure roles
			RoleFactory.getInstance().init(config.getElement("roles"));
			// configure scope types
			ScopeTypeFactory.getInstance().init(config.getElement("scopetypes"));
		} catch (Exception e) {
			logger.error("Configuration inclusion failure", e);
		}
	}

	/**
	 * @param handler
	 * @return
	 * @throws ConfigurationException
	 */
	private Handler init(Element handler) throws ConfigurationException {
		try {
			Handler instance = null;
			String type;
			try {
				type = handler.getAttribute("type");
			} catch (RuntimeException e) {
				type = null;
			}
			String refid;
			try {
				refid = handler.getAttribute("refid");
			} catch (RuntimeException e) {
				refid = null;
			}
			logger.debug("Initializating handler " + handler.getAttribute("id") + ':' + handler.getAttribute("type"));
			if (type != null) {
				instance = (Handler)Class.forName(handler.getAttribute("type")).newInstance();
				instance.setId(handler.getAttribute("id"));
				Map map = new HashMap();
				Iterator params = handler.getElements("param").iterator();
				while (params.hasNext()) {
					Element param = (Element)params.next();
					map.put(param.getAttribute("name"), param.getContent());
				}
				instance.init(map);
			} else if (refid != null) {
				instance = (Handler)this.handlers.get(handler.getAttribute("refid"));
			} else {
				throw new ConfigurationException("Error parsing configuration file");
			}
			return instance;
		} catch (Exception e) {
			logger.error("Initialization failure on " + handler.getAttribute("type"), e);
			return null;
		}
	}

	/**
	 * Allow users to self register in the system. The self registrator
	 * procedure could be customized through the
	 * <code>RegistrationStrategy</code> interface.
	 * 
	 * @param user the user to be registered
	 * @param parameters the map of properties to be providen to the custom
	 *        registrator process
	 * @param step TODO documentation
	 * @return
	 * @throws BusinessException if something wrong occurs during the
	 *         registrator process
	 */
	public String register(Map parameters, String step) throws BusinessException {
		String next = step;
		try {
			Iterator handlers = registrator.iterator();
			while (handlers.hasNext() && step.equals(next)) {
				RegistrationHandler handler = (RegistrationHandler)handlers.next();
				next = handler.onRegister(parameters, step);
			}
			return next;
		} catch (Exception e) {
			throw new BusinessException("Registration error", e);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param user
	 * @param parameters
	 * @param step
	 * @return
	 * @throws BusinessException
	 */
	public String updateUser(User user, Map parameters, String step) throws BusinessException {
		String next = step;
		try {
			Iterator handlers = registrator.iterator();
			while (handlers.hasNext() && step.equals(next)) {
				RegistrationHandler handler = (RegistrationHandler)handlers.next();
				next = handler.onUpdate(user, parameters, step);
			}
			return next;
		} catch (Exception e) {
			throw new BusinessException("Exception in registrator handling", e);
		}
	}

	/**
	 * @param securityToken
	 * @param role
	 * @param scope
	 * @param parameters
	 * @return
	 * @throws AuthorizationException
	 * @throws BusinessException
	 */
	public boolean authorize(String securityToken, Role role, Scope scope, Map parameters)
			throws AuthorizationException, BusinessException {
		User user = (User)sessions.get(securityToken);
		if (user == null) {
			throw new AuthenticationException("errors.authentication");
		}
		Iterator privileges = role.getPrivileges().iterator();
		while (privileges.hasNext()) {
			Privilege privilege = (Privilege)privileges.next();
			if (!user.hasPrivilege(privilege, scope)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Logs in a user verifing the providen credentials matches the ones stored
	 * into the database.
	 * 
	 * @param credentials the user credentials which ensure the user is whoever
	 *        he pretends to impersonate.
	 * @return the authenticated user.
	 * @throws BusinessException if something wrong occurs during credentials
	 *         verification.
	 * @throws AuthenticationException if the providen credentials does not
	 *         matches with the stored ones.
	 */
	public User login(Credentials credentials) throws BusinessException, AuthenticationException {
		if (logger.isDebugEnabled()) {
			logger.debug("before login - user: " + User.current().getDisplay());
			logger.debug("login(credentials=" + credentials + ") - start");
		}
		try {
			if (credentials == null || credentials.getUsername() == null
					|| credentials.getUsername().trim().length() == 0) {
				throw new AuthenticationException("Invalid credentials");
			}
			User user = (User)UserFactory.getInstance().findByUsername(credentials.getUsername());
			if (user != null && credentials.getSecret() != null) {
				try {
					Iterator handlers = authenticator.iterator();
					while (handlers.hasNext()) {
						AuthenticationHandler handler = (AuthenticationHandler)handlers.next();
						if (handler != null) {
							handler.onLogin(user, credentials);
						}
					}
				} catch (AuthenticationException ae) {
					throw ae;
				} catch (Exception e) {
					throw new BusinessException(e);
				}
				String securityToken = this.generateSecurityToken();
				user.setSecurityToken(securityToken);
				logger.info("security token " + securityToken + " generated for user " + user.getId());
				Object previous = sessions.put(securityToken, user);
				if (previous != null) {
					throw new BusinessException("Security token collision detected: " + securityToken);
				}
				user.login();
				if (logger.isDebugEnabled()) {
					logger.debug("login(credentials=" + credentials + ") - end");
				}
				UserFactory.getInstance().update(user);
				return user;
			} else {
				throw new AuthenticationException("Invalid credentials");
			}
		} catch (DAOException daoe) {
			throw new AuthenticationException(daoe);
		}
	}

	/**
	 * Logs out a user from the system.
	 * 
	 * @param ticket the previously generated session ticket for the user.
	 * @throws BusinessException if something wrong occurs during the logout
	 *         procedure.
	 */
	public void logout(String securityToken) throws BusinessException {
		if (logger.isDebugEnabled()) {
			logger.debug("logout(securityToken=" + securityToken + ") - start");
		}
		if (securityToken != null && securityToken.trim().length() > 0) {
			User user = (User)sessions.remove(securityToken);
			if (user != null) {
				user.logout();
				try {
					UserFactory.getInstance().update(user);
				} catch (DAOException daoe) {
					throw new AuthenticationException(daoe);
				}
			}
		}
		if (logger.isDebugEnabled()) {
			logger.debug("logout(securityToken=" + securityToken + ") - end");
		}
	}

	/**
	 * Logs out a user from the system.
	 * 
	 * @param user the user to log out.
	 * @throws BusinessException if something wrong occurs during the logout
	 *         procedure.
	 */
	public void logout(User user) throws BusinessException {
		if (logger.isDebugEnabled()) {
			logger.debug("logout(user=" + user + ") - start");
		}
		if (user != null && user != User.GUEST) {
			sessions.remove(user.getSecurityToken());
			user.logout();
			try {
				UserFactory.getInstance().update(user);
			} catch (DAOException daoe) {
				throw new AuthenticationException(daoe);
			}
		}
		if (logger.isDebugEnabled()) {
			logger.debug("logout(user=" + user + ") - end");
		}
	}

	// /**
	// * @param securityToken
	// * @param role
	// * @param scope
	// * @param params
	// * @return
	// * @throws AuthorizationException
	// */
	// public boolean authorize(User user, Privilege privilege, BusinessObject
	// scope, String[] handlersId, Map params)
	// throws AuthorizationException {
	// if (user != null && user.getSecurityToken() != null &&
	// Status.ENABLED.equals(user.getStatus())
	// && sessions.get(user.getSecurityToken()).equals(user)) {
	// Scope sc = null;
	// if (scope != null) {
	// try {
	// sc = Scope.generate(scope);
	// } catch (BusinessException e) {
	// throw new AuthorizationException(e);
	// }
	// }
	// return user.hasPrivilege(privilege, sc);
	// }
	// // FIXME manage handlers
	// return false;
	// }
	/**
	 * TODO documentation
	 * 
	 * @param info
	 * @return
	 * @throws BusinessException
	 */
	public Collection listSubjects(SearchInfo info) throws BusinessException {
		try {
			return SubjectFactory.getInstance().list(info);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @return
	 * @throws BusinessException
	 */
	public Subject findSubject(String id) throws BusinessException {
		try {
			return (Subject)SubjectFactory.getInstance().findByKey(id);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @return
	 * @throws BusinessException
	 */
	public User findUser(String id) throws BusinessException {
		try {
			User user = (User)UserFactory.getInstance().findByKey(Long.valueOf(id));
			user.getGroups();
			user.getPolicy();
			return user;
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param info
	 * @return
	 * @throws BusinessException
	 */
	public Collection listUsers(SearchInfo info) throws BusinessException {
		try {
			return UserFactory.getInstance().list(info);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param info
	 * @return
	 * @throws BusinessException
	 */
	public Collection pageUsers(SearchInfo info) throws BusinessException {
		try {
			return UserFactory.getInstance().page(info);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param idSession
	 * @throws BusinessException
	 */
	public void removeUser(String idSession) throws BusinessException {
		try {
			UserFactory.getInstance().remove(this.findUser(idSession));
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param user
	 * @throws BusinessException
	 */
	public void updateUser(User user) throws BusinessException {
		try {
			UserFactory.getInstance().update(user);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param status
	 * @return
	 * @throws BusinessException
	 */
	public long count(User.Status status) throws BusinessException {
		try {
			return UserFactory.getInstance().count(status);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @return
	 * @throws BusinessException
	 */
	public Role findRole(String id) throws BusinessException {
		return (Role)RoleFactory.getInstance().findByKey(id);
	}

	/**
	 * TODO documentation
	 * 
	 * @param type
	 * @return
	 */
	public Scope.Type findScopeType(Class type) {
		return ScopeTypeFactory.getInstance().findByType(type);
	}

	/**
	 * TODO documentation
	 * 
	 * @param info
	 * @return
	 * @throws BusinessException
	 */
	public Collection listRoles(SearchInfo info) throws BusinessException {
		try {
			return RoleFactory.getInstance().list(info);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param info
	 * @return
	 * @throws BusinessException
	 */
	public Collection pageRoles(SearchInfo info) throws BusinessException {
		try {
			return new CollectionPaginator(RoleFactory.getInstance().list(info));
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @throws BusinessException
	 */
	public void removeRole(String id) throws BusinessException {
		try {
			RoleFactory.getInstance().remove(this.findRole(id));
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param role
	 * @throws BusinessException
	 */
	public void updateRole(Role role) throws BusinessException {
		try {
			RoleFactory.getInstance().update(role);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @return
	 * @throws BusinessException
	 */
	public Group findGroup(String id) throws BusinessException {
		try {
			return (Group)GroupFactory.getInstance().findByKey(id);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param info
	 * @return
	 * @throws BusinessException
	 */
	public Collection listGroups(SearchInfo info) throws BusinessException {
		try {
			return GroupFactory.getInstance().list(info);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param info
	 * @return
	 * @throws BusinessException
	 */
	public Collection pageGroups(SearchInfo info) throws BusinessException {
		try {
			return GroupFactory.getInstance().page(info);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @throws BusinessException
	 */
	public void removeGroup(String id) throws BusinessException {
		try {
			GroupFactory.getInstance().remove(this.findGroup(id));
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param group
	 * @throws BusinessException
	 */
	public void updateGroup(Group group) throws BusinessException {
		try {
			GroupFactory.getInstance().update(group);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param subject
	 * @param scope
	 * @param role
	 * @param handler
	 * @throws BusinessException
	 */
	public void addPermission(Subject subject, Scope scope, Role role, AuthorizationHandler handler)
			throws BusinessException {
		Permission permission = new Permission();
		permission.setSubject(subject);
		permission.setScope(scope);
		permission.setRoleId(role.getId());
		PermissionFactory factory = PermissionFactory.getInstance();
		try {
			factory.update(permission);
		} catch (DAOException e) {
			throw new BusinessException("Exception durin permission saving: " + e.getMessage());
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @return
	 * @throws BusinessException
	 */
	public Permission findPermission(String id) throws BusinessException {
		try {
			return (Permission)PermissionFactory.getInstance().findByKey(id);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @throws BusinessException
	 */
	public void removePermission(String id) throws BusinessException {
		try {
			PermissionFactory.getInstance().remove(this.findPermission(id));
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param info
	 * @return
	 * @throws BusinessException
	 */
	public Collection listPermissions(SearchInfo info) throws BusinessException {
		try {
			return PermissionFactory.getInstance().list(info);
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param info
	 * @return
	 * @throws BusinessException
	 */
	public Collection listPermissions(User user) throws BusinessException {
		try {
			Set result = PermissionFactory.getInstance().listInherited(user);
			result.addAll(User.GUEST.permissions);
			return result;
		} catch (DAOException daoe) {
			throw new BusinessException(daoe);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param permission
	 * @throws BusinessException
	 */
	public void updatePermission(Permission permission) throws BusinessException {
		try {
			PermissionFactory.getInstance().update(permission);
		} catch (DAOException e) {
			throw new BusinessException("error saving permission: " + permission, e);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @return
	 */
	public Map getScopeTypes() {
		return scopes;
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @return
	 */
	protected AuthenticationHandler getAuthenticationHandler(String id) {
		try {
			return (AuthenticationHandler)handlers.get(id);
		} catch (ClassCastException cce) {
			return null;
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @return
	 */
	protected AuthorizationHandler getAuthorizationHandler(String id) {
		try {
			return (AuthorizationHandler)handlers.get(id);
		} catch (ClassCastException cce) {
			return null;
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param id
	 * @return
	 */
	protected AuditHandler getAuditHandler(String id) {
		try {
			return (AuditHandler)handlers.get(id);
		} catch (ClassCastException cce) {
			return null;
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param node
	 * @return
	 */
	public Handler getHandler(String node) {
		return (Handler)this.handlers.get(node);
	}

	/**
	 * TODO documentation
	 * 
	 * @return
	 */
	private final String generateSecurityToken() {
		String token = Long.toHexString(random.nextLong());
		return token;
	}

	/**
	 * TODO documentation
	 * 
	 * @return
	 */
	public Collection getRegistrator() {
		return (Collection)((ArrayList)this.registrator).clone();
	}
}
