/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.smartlab.web.AbstractArchiveAction;
import net.smartlab.web.ActionException;
import net.smartlab.web.BusinessException;
import net.smartlab.web.DynaAction;
import net.smartlab.web.DataAccessObject.SearchInfo;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;

/**
 * TODO Documentation
 * 
 * @author svetrini
 * @author rlogiacco
 */
public class PermissionAction extends DynaAction {

	/**
	 * The <code>domain</code>
	 */
	private Domain domain = Domain.getInstance();


	/**
	 * @see net.smartlab.web.AbstractArchiveAction#remove(org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.apache.struts.action.ActionMapping)
	 */
	public String remove(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws Exception {
		request.setAttribute("subject", domain.findPermission(request.getParameter("subject-id")).getSubject());
		domain.removePermission(request.getParameter("id"));
		return "success";
	}

	/**
	 * @see net.smartlab.web.AbstractArchiveAction#removeAll(org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.apache.struts.action.ActionMapping)
	 */
	public String removeAll(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws Exception {
		String[] ids = AbstractArchiveAction.getListSelection(request);
		if (ids.length > 0) {
			request.setAttribute("subject", domain.findPermission(ids[0]).getSubject());
		} else {
			return "cancel";
		}
		for (int i = 0; i < ids.length; i++) {
			domain.removePermission(ids[i]);
		}
		return "success";
	}

	/**
	 * @see net.smartlab.web.AbstractArchiveAction#search(org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.apache.struts.action.ActionMapping)
	 */
	public String search(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws Exception {
		Collection permissions = domain.listPermissions(AbstractArchiveAction.getSearchInfo(request));
		request.setAttribute("permissions", permissions);
		return "success";
	}

	/**
	 * @see net.smartlab.web.AbstractArchiveAction#select(org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.apache.struts.action.ActionMapping)
	 */
	public String select(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws ActionException {
		try {
			SearchInfo infoAll = new SearchInfo();
			Collection allRoles = domain.listRoles(infoAll);
			request.setAttribute("allRoles", allRoles);
			Collection allSubjects = domain.listSubjects(infoAll);
			request.setAttribute("allSubjects", allSubjects);
			DynaActionForm dform = (DynaActionForm)form;
			String id = dform.getString("id");
			Permission permission = null;
			if (id == null || "".equals(id)) {
				permission = domain.findPermission("" + 0);
				String subjectId = dform.getString("subject-id");
				if (subjectId != null && !"".equals(subjectId)) {
					Subject subject = (Subject)domain.findSubject(subjectId);
					permission.setSubject(subject);
				}
			} else {
				permission = domain.findPermission(id);
			}
			request.setAttribute("permission", permission);
			if (form != null && !super.hasErrors(request)) {
				super.reset(form, request, mapping);
				super.populate(form, permission, request.getLocale());
				dform.set("role-selection", gerRoleBySubject(permission.getSubject()));
				if (permission != null)
					dform.set("role-id", permission.getRoleId());
				if (permission != null && permission.getSubject() != null)
					dform.set("subject-id", "" + permission.getSubject().getId());
			}
		} catch (BusinessException be) {
			throw new ActionException(be.getMessage(), be.getCause());
		}
		return "success";
	}

	/**
	 * @see net.smartlab.web.AbstractArchiveAction#update(org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.apache.struts.action.ActionMapping)
	 */
	public ActionForward update(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws Exception {
		try {
			String permissionId = request.getParameter("id");
			String subjectId = request.getParameter("subject-id");
			String roleId = request.getParameter("role-id");
			Subject subject = domain.findSubject(subjectId);
			if (permissionId == null || "".equals(permissionId)) {
				permissionId = "0";
			}
			Permission permission = domain.findPermission(permissionId);
			permission.setSubject(subject);
			if (!"".equals(roleId)) {
				Role role = domain.findRole(roleId);
				permission.setRoleId(role.getId());
				domain.updatePermission(permission);
			} else {
				domain.removePermission("" + permission.getId());
			}
			ActionForward forward = null;
			if (subject instanceof User) {
				forward = mapping.findForward("user");
				forward.setProperty("subject-id", subjectId);
			} else if (subject instanceof Group) {
				forward = mapping.findForward("user");
				forward.setProperty("subject-id", subjectId);
			}
			return forward;
		} catch (BusinessException be) {
			throw new ActionException(be.getMessage(), be.getCause());
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param subject
	 * @return
	 */
	public static Object[] gerRoleBySubject(Subject subject) {
		if (subject == null) {
			return new Object[0];
		} else {
			Set roles = new HashSet();
			Set permissions = subject.getPermissions();
			for (Iterator iterator = permissions.iterator(); iterator.hasNext();) {
				Permission perm = (Permission)iterator.next();
				roles.add(perm.getRoleId());
			}
			ArrayList result = new ArrayList(roles);
			return result.toArray();
		}
	}
}
