/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import net.smartlab.web.BusinessObjectFactory;
import net.smartlab.web.DAOException;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Expression;

/**
 * @author rlogiacco
 * @author svetrini
 */
public class PermissionFactory extends BusinessObjectFactory {

	/**
	 * Stores the singleton <code>instance</code> for this class.
	 * 
	 * @uml.property name="instance"
	 */
	private static PermissionFactory instance;


	/**
	 * Blocks class instantiation to enforce the singleton pattern.
	 */
	private PermissionFactory() {
		super();
	}

	/**
	 * @return Returns the instance.
	 * @uml.property name="instance"
	 */
	public static synchronized PermissionFactory getInstance() {
		if (instance == null) {
			instance = new PermissionFactory();
		}
		return instance;
	}

	/**
	 * @see net.smartlab.web.BusinessObjectFactory#getMappedClass()
	 */
	public Class getMappedClass() {
		return Permission.class;
	}

	public Set listInherited(User user) throws DAOException {
		Set result = new HashSet();
		if (logger.isDebugEnabled()) {
			logger.debug("list(user +" + user.getId() + ") - start");
		}
		try {
			Session session = super.current();
			Criteria criteria = session.createCriteria(this.getMappedClass());
			criteria.add(Expression.eq("subject.id", new Long(user.getId())));
			Collection first = criteria.list();
			Query query = session.createQuery("select p from User as u left join u.groups as g left join g.permissions as p where u.id=:user");
			query.setLong("user", user.getId());
			Collection second = query.list();
			if (second != null) {
				first.addAll(second);
			}
			result.addAll(first);
			result.remove(null);
			return result;
		} catch (HibernateException he) {
			throw new DAOException("list(user = " + user.getId() + ") - failed", he);
		}
	}

	public Object findByKey(Serializable key) throws DAOException {
		Permission permission = (Permission)super.findByKey(key);
		if (permission != null) {
			permission.getScope();
			permission.getSubject();
		}
		return permission;
	}
}
