/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth;

import java.util.Map;

import net.smartlab.web.BusinessObject;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.builder.CompareToBuilder;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * TODO documentation
 * 
 * @author rlogiacco
 * @author svetrini
 * @uml.stereotype uml_id="Standard::Type"
 */
public class Privilege extends BusinessObject implements Comparable, Cloneable {

	private static final long serialVersionUID = 5093513127253605079L;

	/**
	 * @uml.property name="logger"
	 * @uml.associationEnd
	 */
	protected final Log logger = LogFactory.getLog(Privilege.class);

	public final static String ANY = "*";

	/**
	 * TODO documentation
	 */
	public final static byte READ = 0x01;

	/**
	 * TODO documentation
	 */
	public final static byte WRITE = 0x02;

	/**
	 * TODO documentation
	 */
	public final static byte EXECUTE = 0x04;

	/**
	 * TODO documentation
	 */
	public static final byte NONE = 0x00;

	/**
	 * TODO documentation
	 * 
	 * @uml.property name="resource"
	 */
	private String resource;

	/**
	 * TODO documentation
	 * 
	 * @uml.property name="element"
	 */
	private String element;

	/**
	 * TODO documentation
	 * 
	 * @uml.property name="modifier"
	 */
	private byte modifier;

	/**
	 * TODO documentation
	 * 
	 * @uml.property name="handler"
	 */
	private AuthorizationHandler handler;


	/**
	 * TODO documentation
	 */
	public Privilege() {
		super();
	}

	/**
	 * TODO documentation
	 * 
	 * @param resource
	 * @param element
	 * @param modifier
	 */
	public Privilege(String resource, String element) {
		this(resource, element, null, null);
	}

	/**
	 * TODO documentation
	 * 
	 * @param resource
	 * @param element
	 * @param modifier
	 */
	public Privilege(String resource, String element, String modifier) {
		this(resource, element, modifier, null);
	}

	public Privilege(String resource, String element, byte modifier) {
		this(resource, element, null, null);
		this.modifier = modifier;
	}

	/**
	 * TODO documentation
	 * 
	 * @param resource
	 * @param element
	 * @param modifier
	 */
	public Privilege(String resource, String element, String modifier, String handler) {
		this.resource = resource;
		this.element = element;
		this.setModifier(modifier);
		this.handler = Domain.getInstance().getAuthorizationHandler(handler);
	}

	/**
	 * Returns the resource.
	 * 
	 * @return the resource
	 * @uml.property name="resource"
	 */
	public String getResource() {
		return resource;
	}

	/**
	 * Sets the resource.
	 * 
	 * @param resource the resource to set
	 * @uml.property name="resource"
	 */
	public void setResource(String resource) {
		this.resource = resource;
	}

	/**
	 * Returns the element.
	 * 
	 * @return the element
	 * @uml.property name="element"
	 */
	public String getElement() {
		return element;
	}

	/**
	 * Sets the element.
	 * 
	 * @param element the element to set
	 * @uml.property name="element"
	 */
	public void setElement(String element) {
		this.element = element;
	}

	/**
	 * Returns the modifier.
	 * 
	 * @return the modifier
	 * @uml.property name="modifier"
	 */
	public byte getModifier() {
		return modifier;
	}

	/**
	 * Sets the modifier.
	 * 
	 * @param modifier the modifier to set
	 * @uml.property name="modifier"
	 */
	public void setModifier(byte modifier) {
		this.modifier = modifier;
	}

	/**
	 * Sets the modifier.
	 * 
	 * @param modifier the modifier to set
	 */
	public void setModifier(String modifier) {
		if (modifier != null && !modifier.equals(ANY)) {
			this.modifier = NONE;
			char[] chars = modifier.toCharArray();
			for (int i = 0; i < chars.length; i++) {
				switch (chars[i]) {
					case 'r':
					case 'R':
						this.modifier += READ;
						break;
					case 'w':
					case 'W':
						this.modifier += WRITE;
						break;
					case 'x':
					case 'X':
						this.modifier += EXECUTE;
						break;
					default:
						logger.warn("Invalid access modifier `" + chars[i] + "` on " + this.toString());
				}
			}
		} else {
			this.modifier = READ + WRITE + EXECUTE;
		}
	}

	/**
	 * @return
	 * @uml.property name="handler"
	 */
	public String getHandler() {
		return (handler != null ? handler.getId() : null);
	}

	/**
	 * @param handler
	 * @uml.property name="handler"
	 */
	public void setHandler(String handler) {
		this.handler = Domain.getInstance().getAuthorizationHandler(handler);
	}

	/**
	 * Verifies if the specified privilege represents a match for this one. A
	 * privilege is considered matching if it represents a subset of this one.
	 * 
	 * @param privilege the privilege to compare.
	 */
	public boolean includes(Privilege privilege) {
		if (privilege.resource.equals(this.resource) || this.resource.equals(ANY)) {
			if (privilege.element.equals(this.element) || this.element.equals(ANY)) {
				if ((privilege.modifier & this.modifier) >= privilege.modifier) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Performs privilege check including authorization handler execution.
	 * 
	 * @param resource the resource onto which the match should be performed.
	 * @param element the resource element onto which the match should be
	 *        performed.
	 * @param modifier the access modifier requested for the match.
	 * @param parameters execution parameters to be providen to the
	 *        authorization handler.
	 * @return <code>true</code> only if the providen privilege matches this one
	 *         and if the authorization handler execution has been successful.
	 */
	public boolean check(String resource, String element, byte modifier, Map parameters) {
		return this.check(new Privilege(resource, element, modifier), parameters);
	}

	/**
	 * Performs privilege check including authorization handler execution.
	 * 
	 * @param privilege the privilege to match.
	 * @param parameters execution parameters to be providen to the
	 *        authorization handler.
	 * @return <code>true</code> only if the providen privilege matches this one
	 *         and if the authorization handler execution has been successful.
	 */
	public boolean check(Privilege privilege, Map parameters) {
		return check(User.current(), privilege, parameters);
	}

	/**
	 * TODO documentation
	 * 
	 * @param user
	 * @param privilege
	 * @param parameters
	 * @return
	 */
	public boolean check(User user, Privilege privilege, Map parameters) {
		return this.check(user, privilege, null, parameters);
	}
	
	/**
	 * TODO documentation
	 * 
	 * @param user
	 * @param privilege
	 * @param parameters
	 * @return
	 */
	public boolean check(User user, Privilege privilege, Scope scope, Map parameters) {
		if (this.includes(privilege)) {
			if (handler != null) {
				return handler.onRequest(user, scope, parameters);
			}
			return true;
		}
		return false;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(-910312141, -1655346559).append(this.modifier).append(this.element).append(
				this.resource).toHashCode();
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object object) {
		if (object instanceof Privilege) {
			Privilege privilege = (Privilege)object;
			return new EqualsBuilder().append(this.modifier, privilege.modifier)
					.append(this.element, privilege.element).append(this.resource, privilege.resource).isEquals();
		}
		return false;
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object object) {
		Privilege privilege = (Privilege)object;
		return new CompareToBuilder().append(this.modifier, privilege.modifier).append(this.element, privilege.element)
				.append(this.resource, privilege.resource).toComparison();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this).append("element", element).append("resource", resource).append("modifier",
				modifier).toString();
	}

	/**
	 * @see java.lang.Object#clone()
	 */
	protected Object clone() throws CloneNotSupportedException {
		try {
			return BeanUtils.cloneBean(this);
		} catch (Exception e) {
			logger.error(e);
			return null;
		}
	}
}
