package net.smartlab.web.auth;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import net.smartlab.config.Configuration;
import net.smartlab.config.ConfigurationException;
import net.smartlab.config.Element;
import net.smartlab.web.DataAccessObject;
import net.smartlab.web.auth.Scope.Type;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * TODO documentation
 * 
 * @author svetrini
 */
public class ScopeTypeFactory implements DataAccessObject {

	/**
	 * Logger for this class
	 */
	private static final Log logger = LogFactory.getLog(ScopeTypeFactory.class);

	private static ScopeTypeFactory instance = null;

	/**
	 * @uml.property name="types"
	 * @uml.associationEnd qualifier="key:java.lang.String
	 *                     net.smartlab.web.auth.Scope$Type"
	 */
	private Map types = new HashMap();


	/**
	 * TODO documentation
	 */
	protected ScopeTypeFactory() {
		super();
	}

	/**
	 * TODO documentation
	 * 
	 * @param config
	 * @throws ConfigurationException
	 */
	protected void init(Element config) throws ConfigurationException {
		try {
			Iterator scopeTypes = config.getElements().iterator();
			while (scopeTypes.hasNext()) {
				Element scopeType = (Element)scopeTypes.next();
				Type type = new Type(scopeType.getAttribute("id"), scopeType.getAttribute("type"));
				String handlerId = scopeType.getAttribute("handler");
				AuthorizationHandler handler = Domain.getInstance().getAuthorizationHandler(handlerId);
				type.setHandler(handler);
				this.types.put(scopeType.getAttribute("id"), type);
				if (logger.isDebugEnabled()) {
					logger.debug("type id: " + scopeType.getAttribute("id") + " type : "
							+ scopeType.getAttribute("type"));
				}
			}
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param type
	 * @return
	 */
	public Type findByType(Class type) {
		Iterator values = this.types.values().iterator();
		while (values.hasNext()) {
			Type value = (Type)values.next();
			if (value.getType().equals(type.getName())) {
				return value;
			} else {
				try {
					Class valueType = Class.forName(value.getType());
					if (valueType.isAssignableFrom(type)) {
						logger.debug("findByType(type = " + type + ") - resolved to " + value);
						return value;
					}
				} catch (ClassNotFoundException cnfe) {
					logger.warn("findByType(Class) - unable to resolve scope type", cnfe);
				}
			}
		}
		return null;
	}

	/**
	 * TODO documentation
	 * 
	 * @return
	 */
	public static synchronized ScopeTypeFactory getInstance() {
		if (instance == null) {
			instance = new ScopeTypeFactory();
			try {
				Configuration configuration = Domain.getInstance().getConfiguration();
				instance.init(configuration.getElement("scopetypes"));
			} catch (Exception e) {
				logger.error("Cannot initialize scope types", e);
			}
		}
		return instance;
	}

	/**
	 * @see net.smartlab.web.DataAccessObject#findByKey(java.io.Serializable)
	 */
	public Object findByKey(Serializable key) {
		return (Type)types.get(key);
	}

	/**
	 * @see net.smartlab.web.DataAccessObject#list(net.smartlab.web.DataAccessObject.SearchInfo)
	 */
	public Collection list(SearchInfo info) {
		return types.values();
	}

	/**
	 * @see net.smartlab.web.DataAccessObject#remove(java.lang.Object)
	 */
	public void remove(Object object) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see net.smartlab.web.DataAccessObject#update(java.lang.Object)
	 */
	public void update(Object object) {
		throw new UnsupportedOperationException();
	}
}