/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import javax.servlet.ServletRequest;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import net.smartlab.web.AbstractArchiveAction;
import net.smartlab.web.ActionException;
import net.smartlab.web.BusinessException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 * TODO documentation
 * 
 * @author rlogiacco@users.sourceforge.net
 */
public class UserAction extends AbstractArchiveAction {

	/**
	 * The key used to store and retrieve the user status list.
	 */
	public final static String STATUS_LIST_KEY = "user.status.list";

	/**
	 * The key used to store and retrieve the current user from session.
	 */
	public final static String SESSION_KEY = "net.smartlab.web.auth";

	/**
	 * The local domain
	 * 
	 * @uml.property name="domain"
	 * @uml.associationEnd multiplicity="(1 1)"
	 */
	private Domain domain = Domain.getInstance();

	/**
	 * Logger for this class
	 * 
	 * @uml.property name="logger"
	 * @uml.associationEnd multiplicity="(1 1)"
	 */
	protected final Log logger = LogFactory.getLog(AbstractUserAction.class);


	/**
	 * TODO documentation
	 * 
	 * @param form
	 * @param request
	 * @param response
	 * @param mapping
	 * @return
	 * @throws BusinessException
	 * @throws ActionException
	 */
	public String register(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws BusinessException, ActionException {
		// super.valorize(form, user, request.getLocale());
		return domain.register(request.getParameterMap(), request.getParameter("step"));
	}

	/**
	 * TODO documentation
	 * 
	 * @param form
	 * @param request
	 * @param response
	 * @param mapping
	 * @return
	 * @throws Exception
	 * @throws BusinessException
	 */
	public String login(ActionForm form, HttpServletRequest request, HttpServletResponse response, ActionMapping mapping)
			throws Exception {
		this.logout(form, request, response, mapping);
		if (logger.isDebugEnabled()) {
			logger.debug("login(username = " + request.getParameter("username") + ") - start");
		}
		Credentials credentials = new Credentials();
		super.valorize(form, credentials, request.getLocale());
		// valorize doesn't set the secret property due to overridden method
		credentials.setSecret(request.getParameter("secret"));
		User user = domain.login(credentials);
		if (user == null) {
			return "failure";
		} else {
			request.getSession().setAttribute(UserAction.SESSION_KEY, user);
			if (request.getParameter("remember") != null) {
				Cookie cookie = new Cookie("smartweb-auth", credentials.toString());
				cookie.setMaxAge(15 * 24 * 60 * 60);
				response.addCookie(cookie);
			}
			user.getPolicy();
			return "success";
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param form
	 * @param request
	 * @param response
	 * @param mapping
	 * @return
	 * @throws Exception
	 * @throws BusinessException
	 */
	public String autoLogin(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws BusinessException {
		logger.info("autoLogin() - start");
		Cookie[] cookies = request.getCookies();
		for (int i = 0; i < cookies.length; i++) {
			Cookie cookie = cookies[i];
			if (cookie.getName().equals("smartweb-auth")) {
				String value = cookie.getValue();
				User user = null;
				try {
					user = domain.login(new Credentials(value));
				} catch (IOException e) {
					cookie.setMaxAge(0);
					response.addCookie(cookie);
				}
				if (user == null) {
					return "failure";
				} else {
					// renew the expire time
					request.getSession().setAttribute(UserAction.SESSION_KEY, user);
					cookie.setMaxAge(15 * 24 * 60 * 60);
					response.addCookie(cookie);
					return "success";
				}
			}
		}
		return "disabled";
	}

	/**
	 * TODO documentation
	 * 
	 * @param form
	 * @param request
	 * @param response
	 * @param mapping
	 * @return
	 * @throws BusinessException
	 */
	public String logout(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws BusinessException {
		logger.info("logout() - start");
		domain.logout((User)request.getSession().getAttribute(UserAction.SESSION_KEY));
		request.getSession().invalidate();
		return "success";
	}

	/**
	 * TODO documentation
	 * 
	 * @param form
	 * @param request
	 * @param response
	 * @param mapping
	 * @return
	 * @throws ActionException
	 * @throws BusinessException
	 */
	public String remove(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws BusinessException {
		User user = AbstractUserAction.getUser(request);
		if (user != null) {
			domain.removeUser(Long.toString(user.getId()));
		}
		return "success";
	}

	/**
	 * @see net.smartlab.web.Action#cancel(org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.apache.struts.action.ActionMapping)
	 */
	protected ActionForward cancel(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws Exception {
		super.reset(form, request, mapping);
		return mapping.findForward("cancel");
	}

	/**
	 * @see net.smartlab.web.AbstractArchiveAction#search(org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.apache.struts.action.ActionMapping)
	 */
	public String search(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws Exception {
		Collection users = domain.listUsers(super.getSearchInfo(request));
		request.setAttribute("users", users);
		return "success";
	}

	/**
	 * @see net.smartlab.web.AbstractArchiveAction#select(org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.apache.struts.action.ActionMapping)
	 */
	public String select(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws Exception {
		// the list of statuses is needed to select new status
		request.setAttribute(STATUS_LIST_KEY, User.STATUS_LIST);
		// finally retrieve the user details
		User user = domain.findUser(request.getParameter("id"));
		request.setAttribute("user", user);
		return "success";
	}

	/**
	 * @see net.smartlab.web.AbstractArchiveAction#update(org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.apache.struts.action.ActionMapping)
	 */
	public String update(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws Exception {
		User user = domain.findUser(request.getParameter("id"));
		super.valorize(form, user, request.getLocale());
		user.clearGroups();
		String[] groups = request.getParameterValues("group-selection");
		if (groups != null) {
			for (int i = 0; i < groups.length; i++) {
				user.add(domain.findGroup(groups[i]));
			}
		}
		domain.updateUser(user);
		return "success";
	}

	/**
	 * @see net.smartlab.web.AbstractArchiveAction#removeAll(org.apache.struts.action.ActionForm,
	 *      javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.apache.struts.action.ActionMapping)
	 */
	public String removeAll(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws Exception {
		String[] ids = super.getListSelection(request);
		if (ids != null) {
			for (int i = 0; i < ids.length; i++) {
				domain.removeUser(ids[i]);
			}
		}
		return "success";
	}

	/**
	 * TODO documentation
	 * 
	 * @param form
	 * @param request
	 * @param response
	 * @param mapping
	 * @return
	 * @throws Exception
	 */
	public String groups(ActionForm form, HttpServletRequest request, HttpServletResponse response,
			ActionMapping mapping) throws Exception {
		User user = domain.findUser(request.getParameter("id"));
		request.setAttribute("user", user);
		if (super.getSelection(form, "group-selection") == null) {
			String[] selection = new String[user.getGroups().size()];
			Iterator selected = user.getGroups().iterator();
			for (int i = 0; i < selection.length; i++) {
				selection[i] = Long.toString(((Group)selected.next()).getId());
			}
			super.setSelection(form, "group-selection", selection);
			super.resetSelection(form, "role-selection");
		}
		request.setAttribute("groups", domain.pageGroups(super.getSearchInfo(request)));
		return "success";
	}

	/**
	 * TODO documentation
	 * 
	 * @param form
	 * @param request
	 * @param response
	 * @param mapping
	 * @return
	 * @throws Exception
	 */
	public String roles(ActionForm form, HttpServletRequest request, HttpServletResponse response, ActionMapping mapping)
			throws Exception {
		User user = domain.findUser(request.getParameter("id"));
		request.setAttribute("user", user);
		String[] groups = super.getSelection(form, "group-selection");
		Set inherited = new TreeSet();
		if (groups != null) {
			for (int i = 0; i < groups.length; i++) {
				inherited.addAll(domain.findGroup(groups[i]).getPermissions());
			}
		}
		if (super.getSelection(form, "permission-selection") == null) {
			Collection permissions = new TreeSet(inherited);
			permissions.addAll(user.getPermissions());
			String[] selection = new String[permissions.size()];
			Iterator selected = permissions.iterator();
			for (int i = 0; selected.hasNext(); i++) {
				Role role = (Role)selected.next();
				selection[i] = role.getId();
			}
			super.setSelection(form, "permission-selection", selection);
		}
		request.setAttribute("inherited", inherited);
		request.setAttribute("roles", domain.pageRoles(super.getSearchInfo(request)));
		return "success";
	}

	/**
	 * Returns the currently logged user stored in the presentation tier. This
	 * is the correct method to use on the presentation tier to retrieve the
	 * current user.
	 * 
	 * @param request the request through which the session is accessed.
	 * @return the current user or the <code>User.GUEST</code> if no user available.
	 */
	public static User getUser(ServletRequest request) {
		HttpSession session = ((HttpServletRequest)request).getSession();
		User user = (User)session.getAttribute(SESSION_KEY);
		if (user == null) {
			user = User.GUEST;
		}
		return user;
	}
}
