/**
 * 
 */
package net.smartlab.web.auth.handlers;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import net.smartlab.web.auth.RegistrationHandler;

/**
 * @author svetrini
 *
 */
public abstract class AbstractRegistrationHandler implements RegistrationHandler {
	
	public static final String STATE_REQUEST_INSERT = "STATE_REQUEST_INSERT";
	
	//business constants
	
	protected String id;
	
	
	protected static List getCompleteStateList(){
		return new StateGraph(domain.getRegistrator()).getOrderedStateNodes();
	}


	/**
	 * @see net.smartlab.web.auth.Handler#getId()
	 */
	public String getId() {
		return id;
	}
	
	/**
	 * @see net.smartlab.web.auth.Handler#setId(java.lang.String)
	 */
	public void setId(String id) {
		this.id = id;
	}	
	
	
	protected static String next(String step){
		StateNode state = new StateNode(step);
		return (String) getCompleteStateList().get(getCompleteStateList().indexOf(state) + 1).toString();		
	}
	
	
}

class StateNode{
	protected HashSet ins = new HashSet();
			
	protected final String id;
	
	public StateNode(String id) {
		this.id = id;
	}
	public StateNode(String id, HashSet ins) {
		this(id);
		this.ins.addAll(ins);			
	}
	

	
	public String getId(){
		return this.id;
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof StateNode) {
			StateNode other = (StateNode) obj;
			return (this.getId().equals(other.getId()));				
		}
		return false;
	}
	
	public int hashCode() {			
		return this.getId().hashCode();
	}
	
	// IN NODES
	
	public void addInNode(StateNode in){
		this.ins.add(in);
	}
	
	public void removeInNode(StateNode in){
		this.ins.remove(in);
	}
	
	public void addInNode(HashSet ins){
		this.ins.addAll(ins);
	}
	
	public void removeInNode(List ins){
		this.ins.removeAll(ins);
	}
	
	
	
			
	public HashSet getInNodes(){
		return (HashSet) ins.clone();
	}
	
			
	public int inNodesSize(){
		return this.ins.size();
	}
	
	public String toString() {			
		return this.getId();
	}
	

	
	
}

class StateGraph {
	
	HashMap nodes = new HashMap();
	
	public StateGraph() {
		// NOP
	}
	
	public StateGraph(AbstractRegistrationHandler handler) {
		this.addStateIdSequence(handler.getStates());
	}
	
	public StateGraph(Collection handlers) {
		for (Iterator iterator = handlers.iterator(); iterator.hasNext();) {
			RegistrationHandler handler = (RegistrationHandler) iterator.next();
			this.addStateIdSequence(handler.getStates());			
		}
		
	}
	
	public void addStateIdSequence(List stateIdSequence){
		LinkedList sequence = new LinkedList();
		for (Iterator iterator = stateIdSequence.iterator(); iterator.hasNext();) {
			String stateId = (String) iterator.next();
			StateNode state = new StateNode(stateId);
			if (sequence.size() > 0)
			state.addInNode((StateNode)sequence.getLast());
			sequence.addLast(state);
		}	
		addStateNodes(sequence);
	}
	
	public void addStateIdSequence(String[] stateIdArray){
		addStateIdSequence(Arrays.asList(stateIdArray));				
	}
	
	
	public void addStateNodes(StateGraph graph) {
		this.addStateNodes(graph.getOrderedStateNodes());			
	}

	public void addStateNode(StateNode state){
		if (nodes.containsKey(state.getId())){
			StateNode tmp = (StateNode) nodes.get(state.getId());
			tmp.addInNode(state.getInNodes());				
			this.nodes.put(tmp.getId(), tmp);
			
		}else{
			this.nodes.put(state.getId(), state);				
		}
		
	}
	
	protected void addStateNodes(List states){
		for (Iterator iterator = states.iterator(); iterator.hasNext();) {
			StateNode state = (StateNode) iterator.next();
			this.addStateNode(state);				
		}
	}
	
	public List getOrderedStateNodes(){			
		Map nodesClone = (Map) this.nodes.clone();
		Collection nodesList = (Collection) nodesClone.values();			
		return selectNode(nodesList);				
	}

	protected List selectNode(Collection nodesList) {
		ArrayList result = new ArrayList();
		
		if (nodesList.size() > 0){
			StateNode state = null;
			for (Iterator iterator = nodesList.iterator(); iterator.hasNext();) {
				state = (StateNode) iterator.next();
				if (state.inNodesSize() == 0){					
					break;
				}				
			}
			result.add(state);				
			for (Iterator iterator = nodesList.iterator(); iterator.hasNext();) {
				StateNode tmp = (StateNode) iterator.next();
				tmp.removeInNode(state);				
			}
			nodesList.remove(state);
			result.addAll(selectNode(nodesList));											
		} 			
		return result;
	}
	
	
}



