/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth.handlers;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

import net.smartlab.web.BusinessException;
import net.smartlab.web.auth.User;

/**
 * TODO documentation
 * 
 * @author svetrini
 */
public abstract class AbstractlVerifierRegistrationHandler extends AbstractRegistrationHandler {

	// VerifierRegistrationHandler additional states
	public static final String STATE_SEND_MESSAGE = "STATE_SEND_MESSAGE";

	public static final String STATE_REQUEST_CONFIRM = "STATE_REQUEST_CONFIRM";

	public static final String STATE_CONFIRM = "STATE_CONFIRM";

	// business constants
	protected String id;

	public static final String USER_ID_KEY = "id";


	/**
	 * @see net.smartlab.web.auth.Handler#getId()
	 */
	public String getId() {
		return id;
	}

	/**
	 * @see net.smartlab.web.auth.Handler#setId(java.lang.String)
	 */
	public void setId(String id) {
		this.id = id;
	}

	public String onRegister(Map parameters, String step) throws Exception {
		// try to recover user from map
		String userId = (String)parameters.get(USER_ID_KEY);
		User user = null;
		if (userId != null && !userId.trim().equals("") && !userId.equals("0")) {
			user = domain.findUser(userId);
		}
		if (step.equals(STATE_START) || step.equals(STATE_REQUEST_INSERT))
			step = next(step);
		if (step.equals(STATE_INSERT_UPDATE)) {
			// FIXME only authorized people can update personal informations
			user = updateUser(parameters, user);
			return next(step);
		} else if (step.equals(STATE_SEND_MESSAGE)) {
			if (user == null || user.getId() == 0) {
				throw new BusinessException("Illegal state for step: '" + step + "' user not on db");
			}
			sendMessage(user);
			return next(step);
		} else if (step.equals(STATE_CONFIRM)) {
			if (user == null || user.getId() == 0) {
				throw new BusinessException("Illegal state for step: '" + step + "' user not on db");
			}
			confirm(parameters, user);
			return next(step);
		} else
			return step;
	}

	public String onUpdate(User user, Map props, String step) throws Exception {
		if (step.equals(STATE_INSERT_UPDATE)) {
			user = updateUser(props, user);
		}
		return next(step);
	}

	/**
	 * @param parameters
	 * @param user
	 * @throws BusinessException
	 */
	protected abstract void confirm(Map parameters, User user) throws BusinessException;

	/**
	 * @param user
	 * @throws BusinessException
	 */
	protected abstract void sendMessage(User user) throws BusinessException;

	/**
	 * @param parameters
	 * @param user
	 * @return
	 * @throws BusinessException
	 */
	protected abstract User updateUser(Map parameters, User user) throws BusinessException;

	public List getStates() {
		String[] result = {STATE_START, STATE_REQUEST_INSERT, STATE_INSERT_UPDATE, STATE_SEND_MESSAGE,
				STATE_REQUEST_CONFIRM, STATE_CONFIRM, STATE_END};
		return Arrays.asList(result);
	}
}