/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth.handlers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import net.smartlab.web.auth.AuthenticationHandler;
import net.smartlab.web.auth.AuthorizationException;
import net.smartlab.web.auth.AuthorizationHandler;
import net.smartlab.web.auth.Credentials;
import net.smartlab.web.auth.Domain;
import net.smartlab.web.auth.Handler;
import net.smartlab.web.auth.RegistrationHandler;
import net.smartlab.web.auth.Scope;
import net.smartlab.web.auth.User;

/**
 * TODO documentation
 * 
 * @author rlogiacco
 */
public class ChainHandler implements Handler, AuthenticationHandler, AuthorizationHandler, RegistrationHandler {

	private String id;

	/**
	 * @uml.property name="handlers"
	 * @uml.associationEnd multiplicity="(0 -1)" ordering="false"
	 *                     inverse="chainHandler:net.smartlab.web.auth.Handler"
	 */
	private List handlers = new ArrayList();

	/**
	 * Registration states.
	 */
	private List states = new ArrayList();


	/**
	 * @see net.smartlab.web.auth.Handler#getId()
	 */
	public String getId() {
		return id;
	}

	/**
	 * @see net.smartlab.web.auth.Handler#setId(java.lang.String)
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * @see net.smartlab.web.auth.Handler#init(java.util.Map)
	 */
	public void init(Map parameters) throws Exception {
		StringTokenizer tokenizer = new StringTokenizer((String)parameters.get("nodes"), ",");
		while (tokenizer.hasMoreTokens()) {
			String node = tokenizer.nextToken().trim();
			Handler handler = Domain.getInstance().getHandler(node);
			handlers.add(handler);
		}
	}

	/**
	 * @see net.smartlab.web.auth.AuthenticationHandler#onLogin(net.smartlab.web.auth.User,
	 *      net.smartlab.web.auth.Credentials)
	 */
	public void onLogin(User user, Credentials credential) throws Exception {
		Iterator handlers = this.handlers.iterator();
		while (handlers.hasNext()) {
			Handler handler = (Handler)handlers.next();
			if (handler instanceof AuthenticationHandler) {
				((AuthenticationHandler)handler).onLogin(user, credential);
			}
		}
	}

	/**
	 * @see net.smartlab.web.auth.AuthenticationHandler#onLogout(net.smartlab.web.auth.User)
	 */
	public void onLogout(User user) throws Exception {
		Iterator handlers = this.handlers.iterator();
		while (handlers.hasNext()) {
			Handler handler = (Handler)handlers.next();
			if (handler instanceof AuthenticationHandler) {
				((AuthenticationHandler)handler).onLogout(user);
			}
		}
	}

	/**
	 * @see net.smartlab.web.auth.AuthorizationHandler#onRequest(java.lang.Object[])
	 */
	public boolean onRequest(User user, Scope scope, Map parameters) throws AuthorizationException {
		Iterator handlers = this.handlers.iterator();
		boolean result = true;
		while (result && handlers.hasNext()) {
			Handler handler = (Handler)handlers.next();
			if (handler instanceof AuthenticationHandler) {
				result = ((AuthorizationHandler)handler).onRequest(user, scope, parameters);
			}
		}
		return result;
	}

	/**
	 * @see net.smartlab.web.auth.RegistrationHandler#onRegister(net.smartlab.web.auth.User,
	 *      java.util.Map, java.lang.String)
	 */
	public String onRegister(Map parameters, String step) throws Exception {
		String result = step;
		Iterator handlers = this.handlers.iterator();
		while (handlers.hasNext()) {
			Handler handler = (Handler)handlers.next();
			if (handler instanceof RegistrationHandler) {
				result = ((RegistrationHandler)handler).onRegister(parameters, result);
				// FIXME result overrides previous one
				// TODO serialize the registrations
			}
		}
		return result;
	}

	/**
	 * @see net.smartlab.web.auth.RegistrationHandler#onUpdate(net.smartlab.web.auth.User,
	 *      java.util.Map, java.lang.String)
	 */
	public String onUpdate(User user, Map parameters, String registrationState) throws Exception {
		String result = registrationState;
		Iterator handlers = this.handlers.iterator();
		while (handlers.hasNext()) {
			Handler handler = (Handler)handlers.next();
			if (handler instanceof RegistrationHandler) {
				result = ((RegistrationHandler)handler).onUpdate(user, parameters, result);
				// FIXME result overrides previous one
				// TODO serialize the registrations
			}
		}
		return result;
	}

	public synchronized List getStates() {
		if (states.isEmpty()) {
			Iterator handlers = this.handlers.iterator();
			while (handlers.hasNext()) {
				Handler handler = (Handler)handlers.next();
				if (handler instanceof RegistrationHandler) {
					states.addAll(((RegistrationHandler)handler).getStates());
				}
			}
		}
		return states;
	}

	/**
	 * @see net.smartlab.web.auth.Handler#destroy()
	 */
	public void destroy() throws Exception {
		Iterator handlers = this.handlers.iterator();
		while (handlers.hasNext()) {
			((Handler)handlers.next()).destroy();
		}
	}
}