/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth.handlers;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Random;

import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.el.ELException;
import javax.servlet.jsp.el.FunctionMapper;
import javax.servlet.jsp.el.VariableResolver;

import net.smartlab.web.BusinessException;
import net.smartlab.web.auth.User;
import net.smartlab.web.auth.User.Status;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.el.ExpressionEvaluatorImpl;
import org.apache.commons.el.VariableResolverImpl;
import org.apache.commons.el.parser.ParseException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.mock.MockPageContext;

/**
 * TODO documentation
 * 
 * @author rlogiacco
 * @author svetrini
 */
public class EmailVerifierRegistrationHandler extends AbstractlVerifierRegistrationHandler {

	// message and smtp param
	public static final String RESPONSE_BODY_PARAM = "response.body";

	public static final String RESPONSE_OBJECT_PARAM = "response.object";

	public static final String REQUEST_BODY_PARAM = "request.body";

	public static final String REQUEST_OBJECT_PARAM = "request.object";

	public static final String MAIL_FROM_EMAIL_PARAM = "mail.from.email";

	public static final String MAIL_SMTP_PASS_PARAM = "mail.smtp.pass";

	public static final String MAIL_SMTP_USER_PARAM = "mail.smtp.user";

	public static final String MAIL_SMTP_PORT_PARAM = "mail.smtp.port";

	public static final String MAIL_SMTP_HOST_PARAM = "mail.smtp.host";

	public static final String SYSTEM_NAME_PARAM = "system";

	// business constants
	public static final String TRACE_KEY = "trace";

	public static final String EMAIL_ADDRESS_KEY = "emailAddress";

	// smtp server variables
	private String host;

	private String from;

	private String port;

	private String systemName;

	private String username;

	private String password;

	// message variables
	private Map paramBeans = new HashMap();

	private String reqObject;

	private String reqBody;

	private String resObject;

	private String resBody;


	/**
	 * @param parameters
	 * @param user
	 * @throws BusinessException
	 */
	protected void confirm(Map parameters, User user) throws BusinessException {
		// trace check
		if (parameters.get(TRACE_KEY) != null && !parameters.get(TRACE_KEY).equals("")
				&& parameters.get(TRACE_KEY).equals(user.getProperty(TRACE_KEY))) {
			// OK
			user.setStatus(Status.ENABLED);
			user.removeProperty(TRACE_KEY);
			domain.updateUser(user);
			// send email response
			try {
				String message = generateResponseMessage(user);
				String object = generateResponseObject(user);
				this.sendMail(user.getProperty(EMAIL_ADDRESS_KEY), object, message);
			} catch (Exception e) {
				throw new BusinessException("Error sending Email", e);
			}
		} else {
			throw new BusinessException("Illegal trace, user can't be registered");
		}
	}

	/**
	 * @param user
	 * @throws BusinessException
	 */
	protected void sendMessage(User user) throws BusinessException {
		user.setProperty(TRACE_KEY, generateTrace());
		domain.updateUser(user);
		try {
			// request confirm by email
			String message = generateRequestMessage(user);
			String object = generateRequestObject(user);
			this.sendMail(user.getProperty(EMAIL_ADDRESS_KEY), object, message);
		} catch (Exception e) {
			throw new BusinessException("Error sending Email", e);
		}
	}

	/**
	 * @param parameters
	 * @param user
	 * @return
	 * @throws BusinessException
	 */
	protected User updateUser(Map parameters, User user) throws BusinessException {
		if (user == null || user.getId() == 0) {
			// insert new user in db
			user = new User();
			try {
				BeanUtils.populate(user, parameters);
			} catch (Exception e) {
				throw new BusinessException("Error inserting user", e);
			}
		}
		user.setStatus(Status.PENDING);
		// update user infos in db
		String emailAddress = (String)parameters.get(EMAIL_ADDRESS_KEY);
		user.setProperty(EMAIL_ADDRESS_KEY, emailAddress);
		domain.updateUser(user);
		parameters.put(USER_ID_KEY, "" + user.getId());
		return user;
	}

	private String generateTrace() {
		Random random = new Random();
		String trace = Long.toHexString(random.nextLong());
		return trace;
	}

	private String generateResponseObject(User user) throws ELException, ParseException {
		HashMap beans = new HashMap();
		beans.putAll(this.paramBeans);
		beans.put("user", user);
		return parseParametricText(this.resObject, beans);
	}

	private String generateRequestObject(User user) throws ELException, ParseException {
		HashMap beans = new HashMap();
		beans.putAll(this.paramBeans);
		beans.put("user", user);
		return parseParametricText(this.reqObject, beans);
	}

	private String generateRequestMessage(User user) throws ELException, ParseException {
		HashMap beans = new HashMap();
		beans.putAll(this.paramBeans);
		beans.put("user", user);
		return parseParametricText(this.reqBody, beans);
	}

	private String generateResponseMessage(User user) throws ELException, ParseException {
		HashMap beans = new HashMap();
		beans.putAll(this.paramBeans);
		beans.put("user", user);
		return parseParametricText(this.resBody, beans);
	}

	public void destroy() throws Exception {
		// TODO Auto-generated method stub
	}

	public void init(Map parameters) throws Exception {
		this.systemName = (String)parameters.get(SYSTEM_NAME_PARAM);
		this.host = (String)parameters.get(MAIL_SMTP_HOST_PARAM);
		this.port = (String)parameters.get(MAIL_SMTP_PORT_PARAM);
		this.username = (String)parameters.get(MAIL_SMTP_USER_PARAM);
		this.password = (String)parameters.get(MAIL_SMTP_PASS_PARAM);
		this.from = (String)parameters.get(MAIL_FROM_EMAIL_PARAM);
		this.reqObject = (String)parameters.get(REQUEST_OBJECT_PARAM);
		this.reqBody = (String)parameters.get(REQUEST_BODY_PARAM);
		this.resObject = (String)parameters.get(RESPONSE_OBJECT_PARAM);
		this.resBody = (String)parameters.get(RESPONSE_BODY_PARAM);
		this.paramBeans.putAll(parameters);
	}

	private void sendMail(String to, String subject, String body) throws MessagingException,
			UnsupportedEncodingException {
		Properties properties = System.getProperties();
		// Setup mail server
		properties.put("mail.smtp.auth", "false");
		properties.put("mail.smtp.host", host);
		properties.put("mail.smtp.port", port);
		properties.put("mail.transport.protocol", "smtp");
		properties.put("mail.smtp.starttls.enable", "false");
		MailSender sender = new MailSender(this.from, this.systemName, this.username, body, subject, to, properties);
		Thread t = new Thread(sender);
		t.start();
	}

	private static String parseParametricText(String text, HashMap beans) throws ELException, ParseException {
		String result = "";
		ExpressionEvaluatorImpl evaluator = new ExpressionEvaluatorImpl();
		Class pExpectedType = String.class;
		PageContext pCtx = new tmpPageContext(beans);
		VariableResolver pResolver = new VariableResolverImpl(pCtx);
		FunctionMapper functions = null;
		result = (String)evaluator.evaluate(text, pExpectedType, pResolver, functions);
		logger.info(result);
		return result;
	}


	private static class tmpPageContext extends MockPageContext {

		public tmpPageContext(HashMap attributes) {
			super();
			super.attributes = attributes;
		}
	}
}

class MailSender implements Runnable {

	static final Log logger = LogFactory.getLog(MailSender.class);

	// smtp server variables
	private String from;

	private String systemName;

	private String username;

	private String body;

	private String subject;

	private String to;

	private Properties properties;


	public void run() {
		try {
			// Setup authentication
			Authenticator authenticator = null;
			if (username != null) {
				authenticator = new Authenticator() {
					// FIXME
				};
			}
			// Get session
			Session session = Session.getDefaultInstance(properties, authenticator);
			// session.setDebug(false);
			// Define message
			MimeMessage message = new MimeMessage(session);
			message.setContent(body, "text/html");
			logger.trace("message content: " + body);
			message.setFrom(new InternetAddress(from, systemName));
			message.addRecipient(Message.RecipientType.TO, new InternetAddress(to));
			message.setSubject(subject);
			// Send message
			Transport.send(message);
		} catch (Exception e) {
			logger.error("Error sending email", e);
		}
	}

	public MailSender(String from, String systemName, String username, String body, String subject, String to,
			Properties properties) {
		super();
		this.from = from;
		this.systemName = systemName;
		this.username = username;
		this.body = body;
		this.subject = subject;
		this.to = to;
		this.properties = properties;
	}
}