/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth.handlers;

import java.util.Map;

import net.smartlab.web.auth.AuthenticationException;
import net.smartlab.web.auth.AuthenticationHandler;
import net.smartlab.web.auth.Credentials;
import net.smartlab.web.auth.User;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * TODO documentation
 * 
 * @author rlogiacco
 */
public class PasswordAuthenticationHandler implements AuthenticationHandler {

	private final static Log logger = LogFactory.getLog(PasswordAuthenticationHandler.class);

	/**
	 * @uml.property name="id"
	 */
	private String id;


	/**
	 * @see net.smartlab.web.auth.Handler#getId()
	 * @uml.property name="id"
	 */
	public String getId() {
		return id;
	}

	/**
	 * @see net.smartlab.web.auth.Handler#setId(java.lang.String)
	 * @uml.property name="id"
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * @see net.smartlab.web.auth.Handler#init(java.util.Map)
	 */
	public void init(Map parameters) throws Exception {
		// nothing to do
	}

	/**
	 * @see net.smartlab.web.auth.AuthenticationHandler#onLogin(net.smartlab.web.auth.User,
	 *      net.smartlab.web.auth.Credentials)
	 */
	public void onLogin(User user, Credentials credentials) throws AuthenticationException {
		if (logger.isDebugEnabled()) {
			logger.debug("onLogin() - start");
		}
		String password = new String(credentials.getSecret());
		if (user.getPassword().equals(password)) {
			if (logger.isTraceEnabled()) {
				logger.trace("   password match");
			}
			return;
		}
		throw new AuthenticationException("Invalid credentials");
	}

	/**
	 * @see net.smartlab.web.auth.AuthenticationHandler#onLogout(net.smartlab.web.auth.User)
	 */
	public void onLogout(User user) throws AuthenticationException {
		if (user == null || user.getUsername() == null) {
			throw new AuthenticationException();
		}
	}

	/**
	 * @see net.smartlab.web.auth.Handler#destroy()
	 */
	public void destroy() throws Exception {
		// nothing to do
	}
}
